from nacl.encoding import Base64Encoder, HexEncoder
from nacl.public import Box, PrivateKey, PublicKey
from pydantic import BaseModel, SecretStr


class KeyPair(BaseModel):
    private_key: SecretStr
    public_key: SecretStr


class Crypt:
    def __init__(self, private_key: bytes, msg_encoder=Base64Encoder, key_encoder=HexEncoder):
        self.msg_encoder = msg_encoder
        self.key_encoder = key_encoder
        self._private_key = PrivateKey(private_key, encoder=self.key_encoder)

    @property
    def public_key(self) -> bytes:
        return self._private_key.public_key._public_key

    @property
    def encoded_public_key(self) -> bytes:
        return self.key_encoder.encode(self.public_key)

    @classmethod
    def generate(cls, encoder=HexEncoder) -> KeyPair:
        private_key = PrivateKey.generate()
        private_key_encoded = private_key.encode(encoder)
        public_key_encoded = private_key.public_key.encode(encoder)
        return KeyPair(private_key=private_key_encoded, public_key=public_key_encoded)

    def encrypt(self, msg: bytes, public_key: bytes) -> str:
        secret = Box(self._private_key, PublicKey(public_key, encoder=self.key_encoder))
        encrypted = secret.encrypt(msg)
        return self.msg_encoder.encode(encrypted).decode()

    def decrypt(self, msg: bytes, public_key: bytes) -> bytes:
        decoded = self.msg_encoder.decode(msg)
        secret = Box(self._private_key, PublicKey(public_key, encoder=self.key_encoder))
        return secret.decrypt(decoded)



import json
import re
from functools import partial

import click
import questionary

from monstermash.crypt import Crypt

red = partial(click.style, fg='red')
blue = partial(click.style, fg='blue')
green = partial(click.style, fg='green')
orange = partial(click.style, fg='orange')


def open_file(file):
    if re.search('.json$', file):
        with open(file, 'r') as f:
            data = json.load(f)
    else:
        with open(file, 'r') as f:
            data = f.read()
    return data


@click.group()
def main():
    pass


@main.command()
def generate():
    keys = Crypt.generate()

    click.echo('-----------------')
    click.echo(red('Private Key (keep is secret, keep it safe)'))
    click.echo(red(keys.private_key.get_secret_value()))
    click.echo(blue('Public Key (you can share this one)'))
    click.echo(blue(keys.public_key.get_secret_value()))
    click.echo('-----------------')


@main.command()
def encrypt():
    private_key = questionary.password('Enter your private key:').ask()
    public_key = questionary.text('Enter the public key:').ask()
    crypt = Crypt(private_key.encode())

    input_type = questionary.select(
        'What type of data do you want to encrypt?', choices=['file', 'text']
    ).ask()

    if input_type == 'file':
        file = questionary.text('Enter the file path:').ask()
        data = open_file(file)
    else:
        data = questionary.password('Enter (or paste) the text to encrypt:').ask()

    encrypted = crypt.encrypt(data.encode(), public_key)
    click.echo(green(encrypted))


@main.command()
def decrypt():
    private_key = questionary.password('Enter your private key:').ask()
    public_key = questionary.text('Enter the public key:').ask()
    crypt = Crypt(private_key.encode())

    data = questionary.text('Enter the encrypted text:').ask()

    decrypted = crypt.decrypt(data.encode(), public_key)
    click.echo(green(decrypted.decode()))


if __name__ == '__main__':
    main()


# 4532a7b115672e99f8d37d45e7635d98065745cd0f74676156372273b0c1119e
# 8305e52593f97fb78356d7d683e7cfbe47bb842af468bc40438efe05d281210f

# e5W7/IIULX4E0KFkyoboXPulIy8w6hGsCmJxYV4yMxIIsHzZwnnlE1oY9tHW



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
from setuptools import find_packages, setup

with open('requirements.txt') as f:
    requirements = f.read().splitlines()


with open('requirements/test.txt') as f:
    test_requirements = f.read().splitlines()


setup(
    author='Mitchell Lisle',
    author_email='m.lisle90@gmail.com',
    description='A Private Key / Public Key Encryption Helper Library',
    install_requires=requirements,
    include_package_data=True,
    keywords='monstermash',
    name='monstermash',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/mitchelllisle/monstermash',
    version='0.2.0',
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'monstermash=monstermash.__main__:main',
        ],
    },
)
